# This file is part of pybliographer
# 
# Copyright (C) 1998 Frederic GOBRY
# Email : gobry@idiap.ch
# 	   
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2 
# of the License, or (at your option) any later version.
#   
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details. 
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
# 
# $Id: Pybliographic.py,v 1.4 1999/07/30 12:34:14 gobry Exp $

from Pyblio.Base import *
import string, os.path
from gnome.ui import *
from gtk import *
import GtkExtra, GDK, GTK

from Pyblio.Open import bibopen, bibwrite
from Pyblio.GnomeUI.Database import *
from Pyblio.GnomeUI.Entry import *
from Pyblio.GnomeUI.Search import *
    
def popup_add (menu, item, action, argument = None):
    """ Add a new menu entry """
    tmp = GtkMenuItem (item)
    tmp.connect ("activate", action, argument)
    
    tmp.show ()
    menu.append (tmp)

class Pybliographic (GnomeApp):

    def __init__ (self):
        GnomeApp.__init__ (self, "Pybliographer", "Pybliographer")
        self.connect("destroy", self.quit)
        self.connect("delete_event", self.quit)

        self.set_title("Pybliographic")
        
        self.main_box = GtkVBox()
        self.set_contents (self.main_box)
        self.main_box.show()
        
        file_menu = [
            UIINFO_ITEM_STOCK ('New', None, self.file_new, STOCK_MENU_NEW),
            UIINFO_ITEM_STOCK ('Open', None, self.file_open, STOCK_MENU_OPEN),
            UIINFO_ITEM_STOCK ('Save', None, self.file_save, STOCK_MENU_SAVE),
            UIINFO_ITEM_STOCK ('Save As...', None, self.file_save_as,
                               STOCK_MENU_SAVE_AS),
            UIINFO_SEPARATOR,
            UIINFO_ITEM_STOCK ('Quit', None, self.file_exit, STOCK_MENU_QUIT),
            ]
        
        help_menu = [
            UIINFO_ITEM_STOCK('About...', None, self.help_about,
                              STOCK_MENU_ABOUT),
            ]

        edit_menu = [
            UIINFO_ITEM_STOCK('Copy', None, self.edit_copy, STOCK_MENU_COPY),
            UIINFO_ITEM_STOCK('Cut', None, self.edit_cut, STOCK_MENU_CUT),
            UIINFO_ITEM_STOCK('Paste', None, self.edit_paste,
                              STOCK_MENU_PASTE),
            UIINFO_SEPARATOR,
            UIINFO_ITEM_STOCK('New...', None, self.edit_new, STOCK_MENU_NEW),
            UIINFO_ITEM('Edit...', None, self.edit_entry, None),
            UIINFO_ITEM_STOCK('Delete', None, self.edit_delete,
                              STOCK_MENU_TRASH),
            UIINFO_SEPARATOR,
            UIINFO_ITEM_STOCK('Search...', None, self.edit_search,
                              STOCK_MENU_SEARCH),
            ]

        config_menu = [
            UIINFO_ITEM_STOCK('Preferences...', None, self.configure_prefs,
                              STOCK_MENU_PREF),
            ]
        
        menus = [
            UIINFO_SUBTREE('File', file_menu),
            UIINFO_SUBTREE('Edit', edit_menu),
            UIINFO_SUBTREE('Configure', config_menu),
            UIINFO_SUBTREE('Help', help_menu)
            ]
        
        toolbar = [
            UIINFO_ITEM_STOCK('Open', None, self.file_open,
                              STOCK_PIXMAP_OPEN),
            UIINFO_ITEM_STOCK('Save', None, self.file_save,
                              STOCK_PIXMAP_SAVE),
            UIINFO_SEPARATOR,
            UIINFO_ITEM_STOCK('Search', None, self.edit_search,
                              STOCK_PIXMAP_SEARCH),
            UIINFO_SEPARATOR,
            UIINFO_ITEM_STOCK('Exit', None, self.file_exit,
                              STOCK_PIXMAP_QUIT),
            ]
        
        self.create_menus (menus)
        self.create_toolbar (toolbar)

        # Put information in Paned windows
        paned = GtkVPaned ()
        paned.show ()

        # The List of data
        self.database = Database ()
        holder = GtkScrolledWindow ()
        holder.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        holder.add (self.database)
        holder.show ()

        self.database.connect ("button_press_event", self.popup_menu)
        self.database.connect ("select_row", self.update_display)
        
        paned.add1 (holder)
        self.database.set_usize (600,300)
        self.database.show ()

        # La zone de texte du bas
        self.display = GtkText ()
        self.display.set_word_wrap (1)
        self.display.set_usize (500,200)
        self.display.show ()
        holder = GtkScrolledWindow ()
        holder.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        holder.add (self.display)
        holder.show ()
        
        paned.add2 (holder)

        self.main_box.pack_start (paned, expand=TRUE)

        self.status = GtkStatusbar ()
        self.set_statusbar (self.status)
        self.status.show ()
        self.status.push (1, "Welcome to Pybliographic")

        # Le popup menu
        self.menu = GtkMenu ()
        popup_add (self.menu, "New...",  self.edit_new)
        popup_add (self.menu, "Edit...", self.edit_entry)
        popup_add (self.menu, "Delete",  self.edit_delete)
        self.menu.show ()

        colormap = self.get_colormap ()
        self.color = {}
        self.color ['red'] = colormap.alloc ("red")
        self.color ['blue'] = colormap.alloc ("blue")

        self.directory = './'
        
    # --------------------------------------------------
        
    def quit (self, *arg):
        mainquit ()
    
    # --------------------------------------------------

    def file_new (self, *arg):
        self.quit ()
    
    # --------------------------------------------------

    def open (self, file):
        """ open a database """

        try:
            self.database.set (bibopen (file))
        except IOError, err:
            dialog = GnomeDialog ('Open Error', STOCK_BUTTON_CLOSE)
            dialog.button_connect (1, GnomeDialog.close)
            dialog.set_usize (500, 300)
            
            text = GtkText ()
            text.insert (None, None, None,
                         "While opening, the following errors occured:\n\n")
            text.insert (None, self.color ['red'], None, str (err))

            holder = GtkScrolledWindow ()
            holder.set_policy (POLICY_AUTOMATIC, POLICY_AUTOMATIC)
            holder.add (text)

            dialog.vbox.pack_start (holder)
            
            dialog.show_all ()
            dialog.run_and_close ()
            return
        
        self.status.push (1, "Database `%s' with %d entries"
                          % (file, len (self.database.data.keys ())))

    def file_open (self, *arg):
        """ callback for the Open button """
        
        win = GtkExtra._FileSelection (TRUE)
        win.set_title ("Open")
        win.set_filename (self.directory)
        
        win.show()
        mainloop()

        file = win.ret
        
        if file:
            self.directory = os.path.split (file) [0] + '/'
            self.open (file)

        
    # --------------------------------------------------

    def file_exit (self, *arg):
        self.quit ()

    # --------------------------------------------------

    def file_save (self, *arg):
        pass

    # --------------------------------------------------

    def file_save_as (self, *arg):
        pass

    # --------------------------------------------------
    # --------------------------------------------------

    def edit_copy (self, * arg):
        pass

    # --------------------------------------------------

    def edit_cut (self, * arg):
        pass

    # --------------------------------------------------

    def edit_paste (self, * arg):
        pass
    
    # --------------------------------------------------

    def edit_search (self, * arg):
        search = Search (self.database)
        search.show_all ()
            
    # --------------------------------------------------

    def edit_new (self, *arg):
        pass

    # --------------------------------------------------

    def edit_entry (self, *arg):
        selection = self.database.selection
        if not selection: return
        
        e = selection [0]
        test = GtkWindow ()
        ve = Entry (e)
        ve.show ()
        test.add (ve)
        test.show ()

    # --------------------------------------------------

    def edit_delete (self, *arg):
        selection = self.database.selection
        if not selection: return

    # --------------------------------------------------

    def configure_prefs (self, * arg):
        pass

    # --------------------------------------------------

    def popup_menu (self, *arg):
        clist, event, = arg

        if (event.type == GDK._2BUTTON_PRESS and event.button == 1):
            # Always select the item below the cursor
            couple = self.database.get_selection_info (event.x, event.y)
            if couple:
                self.database.select_row (couple [0], couple [1])
                self.edit_entry (arg)
            
        if (event.type == GDK.BUTTON_PRESS and event.button == 3):
            # Always select the item below the cursor
            couple = self.database.get_selection_info (event.x, event.y)
            if couple:
                self.database.select_row (couple [0], couple [1])
                
            self.menu.popup (None, None, None, event.button, event.time)

    # --------------------------------------------------

    def update_display (self, *arg):
    
        e = self.database.selection
        
        if e:
            e = e [0]
            
            # Display this entry
            self.display.freeze ()
            self.display.delete_text (0, self.display.get_point ())

            entry = e.type
                
            self.display.insert (None, self.color['blue'], None, entry.name)
            self.display.insert_defaults (" ["+ str (e.name) + "]\n\n")
            
            dico = e.keys ()

            # Search the longest field
            mlen = 0
            for f in dico:
                mlen = max (mlen, len (f))

            for f in entry.fields:
                
		field = string.lower (f.name)
                
		if e.has_key (field):
                    sp = ' ' * (mlen - len (f.name))
                    self.display.insert (None, self.color ['red'], None,
                                         f.name + ": " + sp)
                    self.display.insert_defaults (str (e [field]) + "\n")
                    dico.remove (field)


            self.display.insert_defaults ("\n")
            
            for f in dico:
                sp = ' ' * (mlen - len (f))
                self.display.insert (None, self.color['red'], None,
                                     f + ": " + sp)
                self.display.insert_defaults (str (e [f]) + "\n")

            self.display.thaw ()

            
    # --------------------------------------------------

    def help_about (self, *arg):
        GnomeAbout('Pybliographic', '0.0',
                   'This program is copyrighted under the GNU GPL',
                   ['Frdric Gobry'],
                   'Gnome interface to the Pybliographer system').show()

